@@ -27,26 +27,34 @@
 #include <KToolInvocation>
 #include <KInputDialog>
 #include <KMessageBox>
+#include <KAction>
+
 
-#include <KTp/Models/accounts-model.h>
-#include <KTp/Models/contact-model-item.h>
-#include <KTp/Models/proxy-tree-node.h>
-#include <KTp/Models/groups-model-item.h>
-#include <KTp/Models/groups-model.h>
 #include <KTp/text-parser.h>
+#include <KTp/Widgets/notificationconfigdialog.h>
+#include <KTp/contact-info-dialog.h>
+#include <KTp/types.h>
+#include <KTp/Models/contacts-model.h>
 
 #include <TelepathyQt/ContactManager>
+#include <TelepathyQt/Account>
+#include <TelepathyQt/PendingOperation>
+
+#include <TelepathyLoggerQt4/Entity>
+#include <TelepathyLoggerQt4/LogManager>
+#include <TelepathyLoggerQt4/Init>
 
 #include "dialogs/remove-contact-dialog.h"
-#include "dialogs/contact-info.h"
 
 #include "contact-list-widget_p.h"
-#include <KTp/Models/accounts-filter-model.h>
 
 ContextMenu::ContextMenu(ContactListWidget *mainWidget)
     : QObject(mainWidget)
 {
     m_mainWidget = mainWidget;
+
+    Tpl::init();
+    m_logManager = Tpl::LogManager::instance();
 }
 
 
@@ -58,6 +66,7 @@ ContextMenu::~ContextMenu()
 void ContextMenu::setAccountManager(const Tp::AccountManagerPtr &accountManager)
 {
     m_accountManager = accountManager;
+    m_logManager->setAccountManagerPtr(accountManager);
 }
 
 KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
@@ -72,14 +81,14 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
 
     m_currentIndex = index;
 
-    Tp::ContactPtr contact = index.data(AccountsModel::ItemRole).value<ContactModelItem*>()->contact();
+    KTp::ContactPtr contact = index.data(KTp::ContactRole).value<KTp::ContactPtr>();
 
     if (contact.isNull()) {
         kDebug() << "Contact is nulled";
         return 0;
     }
 
-    Tp::AccountPtr account = m_mainWidget->d_ptr->model->accountForContactItem(index.data(AccountsModel::ItemRole).value<ContactModelItem*>());
+    Tp::AccountPtr account = index.data(KTp::AccountRole).value<Tp::AccountPtr>();
 
     if (account.isNull()) {
         kDebug() << "Account is nulled";
@@ -96,7 +105,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
     connect(action, SIGNAL(triggered(bool)),
             SLOT(onStartTextChatTriggered()));
 
-    if (index.data(AccountsModel::TextChatCapabilityRole).toBool()) {
+    if (index.data(KTp::ContactCanTextChatRole).toBool()) {
         action->setEnabled(true);
     }
 
@@ -112,7 +121,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
     connect(action, SIGNAL(triggered(bool)),
             SLOT(onStartAudioChatTriggered()));
 
-    if (index.data(AccountsModel::AudioCallCapabilityRole).toBool()) {
+    if (index.data(KTp::ContactCanAudioCallRole).toBool()) {
         action->setEnabled(true);
     }
 
@@ -122,7 +131,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
     connect(action, SIGNAL(triggered(bool)),
             SLOT(onStartVideoChatTriggered()));
 
-    if (index.data(AccountsModel::VideoCallCapabilityRole).toBool()) {
+    if (index.data(KTp::ContactCanVideoCallRole).toBool()) {
         action->setEnabled(true);
     }
 
@@ -132,7 +141,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
     connect(action, SIGNAL(triggered(bool)),
             SLOT(onStartFileTransferTriggered()));
 
-    if (index.data(AccountsModel::FileTransferCapabilityRole).toBool()) {
+    if (index.data(KTp::ContactCanFileTransferRole).toBool()) {
         action->setEnabled(true);
     }
 
@@ -142,14 +151,31 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
     connect(action, SIGNAL(triggered(bool)),
             SLOT(onStartDesktopSharingTriggered()));
 
-    if (index.data(AccountsModel::DesktopSharingCapabilityRole).toBool()) {
+    if (index.data(KTp::ContactTubesRole).toStringList().contains(QLatin1String("rfb"))) {
         action->setEnabled(true);
     }
 
+    action = menu->addAction(i18n("Open Log Viewer..."));
+    action->setIcon(KIcon("documentation"));
+    action->setDisabled(true);
+    connect(action, SIGNAL(triggered(bool)),
+            SLOT(onOpenLogViewerTriggered()));
+
+    Tpl::EntityPtr entity = Tpl::Entity::create(contact, Tpl::EntityTypeContact);
+    if (m_logManager->exists(account, entity, Tpl::EventTypeMaskText)) {
+        action->setEnabled(true);
+    }
+
+    menu->addSeparator();
+    action = menu->addAction(KIcon("dialog-information"), i18n("Configure Notifications ..."));
+    action->setEnabled(true);
+    connect(action, SIGNAL(triggered()),
+                           SLOT(onNotificationConfigureTriggered()));
+
     // add "goto" submenu for navigating to links the contact has in presence message
     // first check to see if there are any links in the contact's presence message
     QStringList contactLinks;
-    QString presenceMsg = index.data(AccountsModel::PresenceMessageRole).toString();
+    QString presenceMsg = index.data(KTp::ContactPresenceMessageRole).toString();
     if (presenceMsg.isEmpty()) {
         contactLinks = QStringList();
     } else {
@@ -170,8 +196,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
 
     menu->addSeparator();
 
-    //FIXME add method to modelFilter bool isUsingGroupModel();
-    if (m_mainWidget->d_ptr->modelFilter->sourceModel() == m_mainWidget->d_ptr->groupsModel) {
+    if (m_mainWidget->d_ptr->model->groupMode() == KTp::ContactsModel::GroupGrouping) {
         // remove contact from group action, must be QAction because menu->addAction returns QAction
         QAction *groupRemoveAction = menu->addAction(KIcon(), i18n("Remove Contact From This Group"));
         connect(groupRemoveAction, SIGNAL(triggered(bool)), this, SLOT(onRemoveContactFromGroupTriggered()));
@@ -211,7 +236,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
 
     menu->addSeparator();
 
-    
+
     if (contact->manager()->canRequestPresenceSubscription()) {
         if (contact->subscriptionState() != Tp::Contact::PresenceStateYes) {
             action = menu->addAction(i18n("Re-request Contact Authorization"));
@@ -226,7 +251,7 @@ KMenu* ContextMenu::contactContextMenu(const QModelIndex &index)
     }
 
     action = menu->addSeparator(); //prevent two seperators in a row
-        
+
     if (contact->isBlocked()) {
         action = menu->addAction(i18n("Unblock Contact"));
         connect(action, SIGNAL(triggered(bool)), SLOT(onUnblockContactTriggered()));
@@ -260,12 +285,10 @@ KMenu* ContextMenu::groupContextMenu(const QModelIndex &index)
 
     m_currentIndex = index;
 
-    GroupsModelItem *groupItem = index.data(AccountsModel::ItemRole).value<GroupsModelItem*>();
-
-    Q_ASSERT(groupItem);
+    const QString groupName = index.data(Qt::DisplayRole).toString();
 
     KMenu *menu = new KMenu();
-    menu->addTitle(groupItem->groupName());
+    menu->addTitle(groupName);
 
     //must be QAction, because menu->addAction returns QAction, otherwise compilation dies horribly
     QAction *action = menu->addAction(i18n("Rename Group..."));
@@ -285,13 +308,14 @@ KMenu* ContextMenu::groupContextMenu(const QModelIndex &index)
 
 void ContextMenu::onRemoveContactFromGroupTriggered()
 {
-    QString groupName = m_currentIndex.parent().data(GroupsModel::GroupNameRole).toString();
-    ContactModelItem *contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
+    if (m_currentIndex.parent().data(KTp::RowTypeRole).toUInt() != KTp::GroupRowType) {
+        return;
+    }
 
-    Q_ASSERT(contactItem);
-    Tp::ContactPtr contact =  contactItem->contact();
+    const QString groupName = m_currentIndex.parent().data(Qt::DisplayRole).toString();
+    Tp::ContactPtr contact =  m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
 
-    Tp::PendingOperation* operation = contact->removeFromGroup(groupName);
+    Tp::PendingOperation *operation = contact->removeFromGroup(groupName);
 
     if (operation) {
         connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
@@ -311,11 +335,12 @@ void ContextMenu::onShowInfoTriggered()
         return;
     }
 
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    if (item) {
-        QWeakPointer<ContactInfo> contactInfoDialog = new ContactInfo(item->contact(), m_mainWidget);
-        contactInfoDialog.data()->setAttribute(Qt::WA_DeleteOnClose);
-        contactInfoDialog.data()->show();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    if (account && contact) {
+        KTp::ContactInfoDialog* contactInfoDialog = new KTp::ContactInfoDialog(account, contact, m_mainWidget);
+        contactInfoDialog->setAttribute(Qt::WA_DeleteOnClose);
+        contactInfoDialog->show();
     }
 }
 
@@ -326,22 +351,21 @@ void ContextMenu::onStartTextChatTriggered()
         return;
     }
 
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    if (item) {
-        m_mainWidget->startTextChannel(item);
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
+
+    if (contact && account) {
+        m_mainWidget->startTextChannel(account, contact);
     }
 }
 
 void ContextMenu::onStartAudioChatTriggered()
 {
-    if (!m_currentIndex.isValid()) {
-        kDebug() << "Invalid index provided.";
-        return;
-    }
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
 
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    if (item) {
-        m_mainWidget->startAudioChannel(item);
+    if (contact && account) {
+        m_mainWidget->startAudioChannel(account, contact);
     }
 }
 
@@ -352,9 +376,11 @@ void ContextMenu::onStartVideoChatTriggered()
         return;
     }
 
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    if (item) {
-        m_mainWidget->startVideoChannel(item);
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
+
+    if (contact && account) {
+        m_mainWidget->startVideoChannel(account, contact);
     }
 }
 
@@ -365,9 +391,11 @@ void ContextMenu::onStartFileTransferTriggered()
         return;
     }
 
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    if (item) {
-        m_mainWidget->startFileTransferChannel(item);
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
+
+    if (contact && account) {
+        m_mainWidget->startFileTransferChannel(account, contact);
     }
 }
 
@@ -378,18 +406,32 @@ void ContextMenu::onStartDesktopSharingTriggered()
         return;
     }
 
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    if (item) {
-        m_mainWidget->startDesktopSharing(item);
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
+
+    if (contact && account) {
+        m_mainWidget->startDesktopSharing(account, contact);
+    }
+}
+
+void ContextMenu::onOpenLogViewerTriggered()
+{
+    if (!m_currentIndex.isValid()) {
+      kDebug() << "Invalid index provided.";
+      return;
+    }
+
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::AccountPtr account = m_currentIndex.data(KTp::AccountRole).value<Tp::AccountPtr>();
+
+    if (contact && account) {
+        m_mainWidget->startLogViewer(account, contact);
     }
 }
 
 void ContextMenu::onUnblockContactTriggered()
 {
-    ContactModelItem* item = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    Q_ASSERT(item);
-
-    Tp::ContactPtr contact = item->contact();
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
 
     Tp::PendingOperation *operation = contact->unblock(); //FIXME
     connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
@@ -398,10 +440,7 @@ void ContextMenu::onUnblockContactTriggered()
 
 void ContextMenu::onAddContactToGroupTriggered()
 {
-    ContactModelItem* contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-
-    Q_ASSERT(contactItem);
-    Tp::ContactPtr contact =  contactItem->contact();
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
 
     QAction *action = qobject_cast<QAction*>(sender());
     if (!action) {
@@ -435,10 +474,7 @@ void ContextMenu::onCreateNewGroupTriggered()
                                                  &ok);
 
     if (ok) {
-        ContactModelItem *contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-
-        Q_ASSERT(contactItem);
-        Tp::ContactPtr contact =  contactItem->contact();
+        Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
         Tp::PendingOperation *operation = contact->addToGroup(newGroupName);
 
         connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
@@ -448,27 +484,31 @@ void ContextMenu::onCreateNewGroupTriggered()
 
 void ContextMenu::onRenameGroupTriggered()
 {
-    GroupsModelItem *groupItem = m_currentIndex.data(AccountsModel::ItemRole).value<GroupsModelItem*>();
+    if (m_currentIndex.data(KTp::RowTypeRole).toUInt() != KTp::GroupRowType) {
+        return;
+    }
 
-    Q_ASSERT(groupItem);
+    const QString groupName = m_currentIndex.data(Qt::DisplayRole).toString();
+    const QAbstractItemModel *model = m_currentIndex.model();
 
     bool ok = false;
 
     QString newGroupName = KInputDialog::getText(i18n("New Group Name"),
                                                  i18n("Please enter the new group name"),
-                                                 groupItem->groupName(),
+                                                 groupName,
                                                  &ok);
 
-    if (ok) {
-        for(int i = 0; i < groupItem->size(); i++) {
-            Tp::ContactPtr contact = qobject_cast<ProxyTreeNode*>(groupItem->childAt(i))->data(AccountsModel::ItemRole).value<ContactModelItem*>()->contact();
+    if (ok && groupName != newGroupName) {
+        //loop through all child indexes of m_currentIndex
+        for(int i = 0; i < model->rowCount(m_currentIndex); i++) {
+            Tp::ContactPtr contact = model->index(i, 0 , m_currentIndex).data(KTp::ContactRole).value<KTp::ContactPtr>();
             Q_ASSERT(contact);
 
             Tp::PendingOperation *operation = contact->addToGroup(newGroupName);
             connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
                     m_mainWidget, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
 
-            operation = contact->removeFromGroup(groupItem->groupName());
+            operation = contact->removeFromGroup(groupName);
             connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
                     m_mainWidget, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
         }
@@ -477,30 +517,33 @@ void ContextMenu::onRenameGroupTriggered()
 
 void ContextMenu::onDeleteGroupTriggered()
 {
-    if (m_accountManager.isNull()) {
+    if (m_accountManager.isNull() ||
+        (m_currentIndex.data(KTp::RowTypeRole).toUInt() != KTp::GroupRowType)) {
         return;
     }
 
-    GroupsModelItem *groupItem = m_currentIndex.data(AccountsModel::ItemRole).value<GroupsModelItem*>();
+    const QString groupName = m_currentIndex.data(Qt::DisplayRole).toString();
+    const QAbstractItemModel *model = m_currentIndex.model();
+
 
     if (KMessageBox::warningContinueCancel(m_mainWidget,
                                            i18n("Do you really want to remove group %1?\n\n"
-                                                "Note that all contacts will be moved to group 'Ungrouped'", groupItem->groupName()),
+                                                "Note that all contacts will be moved to group 'Ungrouped'", groupName),
                                            i18n("Remove Group")) == KMessageBox::Continue) {
 
-        for(int i = 0; i < groupItem->size(); i++) {
-            Tp::ContactPtr contact = qobject_cast<ProxyTreeNode*>(groupItem->childAt(i))->data(AccountsModel::ItemRole).value<ContactModelItem*>()->contact();
+        for(int i = 0; i < model->rowCount(m_currentIndex); i++) {
+            Tp::ContactPtr contact = model->index(i, 0 , m_currentIndex).data(KTp::ContactRole).value<KTp::ContactPtr>();
 
             Q_ASSERT(contact);
 
-            Tp::PendingOperation *operation = contact->removeFromGroup(groupItem->groupName());
+            Tp::PendingOperation *operation = contact->removeFromGroup(groupName);
             connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
                     m_mainWidget, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
         }
 
         foreach (const Tp::AccountPtr &account, m_accountManager->allAccounts()) {
             if (account->connection()) {
-                Tp::PendingOperation *operation = account->connection()->contactManager()->removeGroup(groupItem->groupName());
+                Tp::PendingOperation *operation = account->connection()->contactManager()->removeGroup(groupName);
                 connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
                         m_mainWidget, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
             }
@@ -510,10 +553,7 @@ void ContextMenu::onDeleteGroupTriggered()
 
 void ContextMenu::onBlockContactTriggered()
 {
-    ContactModelItem *contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-
-    Q_ASSERT(contactItem);
-    Tp::ContactPtr contact =  contactItem->contact();
+    Tp::ContactPtr contact =  m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
 
     Tp::PendingOperation *operation = contact->block();
     connect(operation, SIGNAL(finished(Tp::PendingOperation*)),
@@ -522,10 +562,7 @@ void ContextMenu::onBlockContactTriggered()
 
 void ContextMenu::onDeleteContactTriggered()
 {
-    ContactModelItem* contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-
-    Q_ASSERT(contactItem);
-    Tp::ContactPtr contact =  contactItem->contact();
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
 
     QList<Tp::ContactPtr>contactList;
     contactList.append(contact);
@@ -554,16 +591,24 @@ void ContextMenu::onDeleteContactTriggered()
 
 void ContextMenu::onRerequestAuthorization()
 {
-    ContactModelItem* contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    Tp::PendingOperation *op = contactItem->contact()->manager()->requestPresenceSubscription(QList<Tp::ContactPtr>() << contactItem->contact());
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::PendingOperation *op = contact->manager()->requestPresenceSubscription(QList<Tp::ContactPtr>() << contact);
     connect(op, SIGNAL(finished(Tp::PendingOperation*)),
             m_mainWidget, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
 }
 
 void ContextMenu::onResendAuthorization()
 {
-    ContactModelItem* contactItem = m_currentIndex.data(AccountsModel::ItemRole).value<ContactModelItem*>();
-    Tp::PendingOperation *op = contactItem->contact()->manager()->authorizePresencePublication(QList<Tp::ContactPtr>() << contactItem->contact());
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+    Tp::PendingOperation *op = contact->manager()->authorizePresencePublication(QList<Tp::ContactPtr>() << contact);
     connect(op, SIGNAL(finished(Tp::PendingOperation*)),
             m_mainWidget, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
 }
+
+void ContextMenu::onNotificationConfigureTriggered()
+{
+    Tp::ContactPtr contact = m_currentIndex.data(KTp::ContactRole).value<KTp::ContactPtr>();
+
+    KTp::NotificationConfigDialog *notificationDialog = new KTp::NotificationConfigDialog(contact, m_mainWidget);
+    notificationDialog->show();
+}
