@@ -1,10 +1,11 @@
 /*
- * This file is part of telepathy-contactslist-prototype
+ * This file is part of ktp-contact-list
  *
  * Copyright (C) 2009-2010 Collabora Ltd. <info@collabora.co.uk>
  *   @Author George Goldberg <george.goldberg@collabora.co.uk>
- * Copyright (C) 2011 Martin Klapetek <martin.klapetek@gmail.com>
  * Copyright (C) 2011 Keith Rusler <xzekecomax@gmail.com>
+ * Copyright (C) 2011-2013 Martin Klapetek <martin.klapetek@gmail.com>
+ * Copyright (C) 2012-2012 David Edmundson <kde@davidedmundson.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,11 +36,12 @@
 #include <TelepathyQt/PendingContacts>
 #include <TelepathyQt/ContactManager>
 #include <TelepathyQt/PendingReady>
+#include <TelepathyQt/TextChannel>
 
-#include <KTp/Models/accounts-model.h>
-#include <KTp/Models/contact-model-item.h>
-#include <KTp/Models/groups-model-item.h>
+#include <KTp/contact-factory.h>
+#include <KTp/types.h>
 #include <KTp/Widgets/add-contact-dialog.h>
+#include <KTp/Widgets/join-chat-room-dialog.h>
 
 #include <KDebug>
 #include <KDialog>
@@ -53,11 +55,12 @@
 #include <KStandardShortcut>
 #include <KNotification>
 #include <KToolInvocation>
+#include <KMenuBar>
+#include <KStandardAction>
 
 #include "ui_main-widget.h"
 #include "account-buttons-panel.h"
 #include "contact-list-application.h"
-#include "dialogs/join-chat-room-dialog.h"
 #include "tooltips/tooltipmanager.h"
 #include "context-menu.h"
 
@@ -69,221 +72,56 @@ bool kde_tp_filter_contacts_by_publication_status(const Tp::ContactPtr &contact)
 }
 
 MainWidget::MainWidget(QWidget *parent)
-    : KMainWindow(parent)
+    : KMainWindow(parent),
+      m_globalMenu(NULL),
+      m_settingsDialog(NULL),
+      m_joinChatRoom(NULL),
+      m_makeCall(NULL),
+      m_contactListTypeGroup(NULL),
+      m_blockedFilterGroup(NULL),
+      m_quitAction(NULL)
 {
-    Tp::registerTypes();
-
     setupUi(this);
+
     m_filterBar->hide();
     setWindowIcon(KIcon("telepathy-kde"));
     setAutoSaveSettings();
-
-    Tp::AccountFactoryPtr  accountFactory = Tp::AccountFactory::create(QDBusConnection::sessionBus(),
-                                                                       Tp::Features() << Tp::Account::FeatureCore
-                                                                       << Tp::Account::FeatureAvatar
-                                                                       << Tp::Account::FeatureCapabilities
-                                                                       << Tp::Account::FeatureProtocolInfo
-                                                                       << Tp::Account::FeatureProfile);
-
-    Tp::ConnectionFactoryPtr connectionFactory = Tp::ConnectionFactory::create(QDBusConnection::sessionBus(),
-                                                                               Tp::Features() << Tp::Connection::FeatureCore
-                                                                               << Tp::Connection::FeatureRosterGroups
-                                                                               << Tp::Connection::FeatureRoster
-                                                                               << Tp::Connection::FeatureSelfContact);
-
-    Tp::ContactFactoryPtr contactFactory = Tp::ContactFactory::create(Tp::Features()  << Tp::Contact::FeatureAlias
-                                                                      << Tp::Contact::FeatureAvatarData
-                                                                      << Tp::Contact::FeatureSimplePresence
-                                                                      << Tp::Contact::FeatureCapabilities
-                                                                      << Tp::Contact::FeatureClientTypes);
-
-    Tp::ChannelFactoryPtr channelFactory = Tp::ChannelFactory::create(QDBusConnection::sessionBus());
-
-    m_accountManager = Tp::AccountManager::create(QDBusConnection::sessionBus(),
-                                                  accountFactory,
-                                                  connectionFactory,
-                                                  channelFactory,
-                                                  contactFactory);
-
-    connect(m_accountManager->becomeReady(), SIGNAL(finished(Tp::PendingOperation*)),
-            this, SLOT(onAccountManagerReady(Tp::PendingOperation*)));
+    setupTelepathy();
 
     KSharedConfigPtr config = KGlobal::config();
     KConfigGroup guiConfigGroup(config, "GUI");
+    setupActions(guiConfigGroup);
+    setupToolBar();
+    setupGlobalMenu();
 
-    m_toolBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
-
-    m_addContactAction = new KAction(KIcon("list-add-user"), i18n("Add New Contacts..."), this);
-
-    m_toolBar->addAction(m_addContactAction);
-
-    m_groupContactsAction = new KDualAction(i18n("Contacts are shown by accounts. Click to show them in groups."),
-                                            i18n("Contacts are shown in groups. Click to show them in accounts."),
-                                            this);
-    m_groupContactsAction->setActiveIcon(KIcon("user-group-properties"));
-    m_groupContactsAction->setInactiveIcon(KIcon("user-group-properties"));
-    m_groupContactsAction->setCheckable(true);
-    m_groupContactsAction->setChecked(true);
-
-    m_toolBar->addAction(m_groupContactsAction);
-
-    m_showOfflineAction = new KDualAction(i18n("Offline contacts are hidden. Click to show them."),
-                                          i18n("Offline contacts are shown. Click to hide them."),
-                                          this);
-    m_showOfflineAction->setActiveIcon(KIcon("meeting-attending-tentative"));
-    m_showOfflineAction->setInactiveIcon(KIcon("meeting-attending-tentative"));
-    m_showOfflineAction->setCheckable(true);
-    m_showOfflineAction->setChecked(false);
-
-    m_toolBar->addAction(m_showOfflineAction);
-
-    m_sortByPresenceAction = new KDualAction(i18n("List is sorted by name. Click to sort by presence."),
-                                             i18n("List is sorted by presence. Click to sort by name."),
-                                             this);
-    m_sortByPresenceAction->setActiveIcon(KIcon("sort-presence"));
-    m_sortByPresenceAction->setInactiveIcon(KIcon("sort-name"));
-
-    m_toolBar->addAction(m_sortByPresenceAction);
-
-    m_searchContactAction = new KAction(KIcon("edit-find-user"), i18n("Find Contact"), this );
-    m_searchContactAction->setShortcut(KStandardShortcut::find());
-    m_searchContactAction->setCheckable(true);
-    m_searchContactAction->setChecked(false);
-
-    m_toolBar->addAction(m_searchContactAction);
-
-    QWidget *toolBarSpacer = new QWidget(this);
-    toolBarSpacer->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-
-    m_toolBar->addWidget(toolBarSpacer);
-
-    QToolButton *settingsButton = new QToolButton(this);
-    settingsButton->setIcon(KIcon("configure"));
-    settingsButton->setPopupMode(QToolButton::InstantPopup);
-
-    KMenu *settingsButtonMenu = new KMenu(settingsButton);
-    settingsButtonMenu->addAction(i18n("Instant Messaging Settings..."), m_contactsListView, SLOT(showSettingsKCM()));
-
-    QActionGroup *delegateTypeGroup = new QActionGroup(this);
-    delegateTypeGroup->setExclusive(true);
-
-    KMenu *setDelegateTypeMenu = new KMenu(settingsButtonMenu);
-    setDelegateTypeMenu->setTitle(i18n("Contact List Type"));
-    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use Full List"),
-                                                                m_contactsListView, SLOT(onSwitchToFullView())));
-    delegateTypeGroup->actions().last()->setCheckable(true);
-
-    if (guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("full")) {
-        delegateTypeGroup->actions().last()->setChecked(true);
-    }
-
-    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use Normal List"),
-                                                                m_contactsListView, SLOT(onSwitchToCompactView())));
-    delegateTypeGroup->actions().last()->setCheckable(true);
-
-    if (guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("normal")
-        || guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("compact")) { //needed for backwards compatibility
-        delegateTypeGroup->actions().last()->setChecked(true);
-    }
-
-    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use Minimalistic List"),
-                                                                     m_contactsListView, SLOT(onSwitchToMiniView())));
-    delegateTypeGroup->actions().last()->setCheckable(true);
-
-    if (guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("mini")) {
-        delegateTypeGroup->actions().last()->setChecked(true);
-    }
-
-    settingsButtonMenu->addMenu(setDelegateTypeMenu);
-
-    KMenu *setBlockedFilterMenu = new KMenu(settingsButtonMenu);
-    setBlockedFilterMenu->setTitle(i18n("Shown Contacts"));
-
-    QActionGroup *blockedFilterGroup = new QActionGroup(this);
-    blockedFilterGroup->setExclusive(true);
-
-    QString shownContacts = guiConfigGroup.readEntry("shown_contacts", "unblocked");
-
-    blockedFilterGroup->addAction(setBlockedFilterMenu->addAction(i18n("Show all contacts"),
-                                                                  m_contactsListView, SLOT(onShowAllContacts())));
-    blockedFilterGroup->actions().last()->setCheckable(true);
-    if (shownContacts == QLatin1String("all")) {
-        blockedFilterGroup->actions().last()->setChecked(true);
-    }
-
-    blockedFilterGroup->addAction(setBlockedFilterMenu->addAction(i18n("Show unblocked contacts"),
-                                                                  m_contactsListView, SLOT(onShowUnblockedContacts())));
-    blockedFilterGroup->actions().last()->setCheckable(true);
-    if (shownContacts == QLatin1String("unblocked")) {
-        blockedFilterGroup->actions().last()->setChecked(true);
-    }
-
-    blockedFilterGroup->addAction(setBlockedFilterMenu->addAction(i18n("Show blocked contacts"),
-                                                                  m_contactsListView, SLOT(onShowBlockedContacts())));
-    blockedFilterGroup->actions().last()->setCheckable(true);
-    if (shownContacts == QLatin1String("blocked")) {
-        blockedFilterGroup->actions().last()->setChecked(true);
-    }
-
-    settingsButtonMenu->addMenu(setBlockedFilterMenu);
-
+    // Restore window geometry, global/by-account presence, search widget state
+    restoreGeometry(guiConfigGroup.readEntry("window_geometry", QByteArray()));
+    toggleSearchWidget(guiConfigGroup.readEntry("pin_filterbar", true));
     if (guiConfigGroup.readEntry("selected_presence_chooser", "global") == QLatin1String("global")) {
         //hide account buttons and show global presence
         onUseGlobalPresenceTriggered();
     }
 
-    // Restore window geometry
-    restoreGeometry(guiConfigGroup.readEntry("window_geometry", QByteArray()));
-
-    settingsButtonMenu->addAction(i18n("Join Chat Room..."), this, SLOT(onJoinChatRoomRequested()));
-
-    if (!KStandardDirs::findExe("ktp-dialout-ui").isEmpty()) {
-        settingsButtonMenu->addAction(i18n("Make a Call..."), this, SLOT(onMakeCallRequested()));
-    }
-
-    settingsButtonMenu->addSeparator();
-    settingsButtonMenu->addMenu(helpMenu());
-
-    settingsButton->setMenu(settingsButtonMenu);
-
-    m_toolBar->addWidget(settingsButton);
-
     m_contextMenu = new ContextMenu(m_contactsListView);
-
     new ToolTipManager(m_contactsListView);
 
     connect(m_contactsListView, SIGNAL(customContextMenuRequested(QPoint)),
             this, SLOT(onCustomContextMenuRequested(QPoint)));
 
-    connect(m_addContactAction, SIGNAL(triggered(bool)),
-            this, SLOT(onAddContactRequest()));
-
     connect(m_groupContactsAction, SIGNAL(triggered(bool)),
             m_contactsListView, SLOT(toggleGroups(bool)));
-
-    connect(m_searchContactAction, SIGNAL(triggered(bool)),
-            this, SLOT(toggleSearchWidget(bool)));
-
-    if (guiConfigGroup.readEntry("pin_filterbar", true)) {
-        toggleSearchWidget(true);
-        m_searchContactAction->setChecked(true);
-    }
-
     connect(m_showOfflineAction, SIGNAL(toggled(bool)),
             m_contactsListView, SLOT(toggleOfflineContacts(bool)));
+    connect(m_sortByPresenceAction, SIGNAL(activeChanged(bool)),
+            m_contactsListView, SLOT(toggleSortByPresence(bool)));
 
     connect(m_filterBar, SIGNAL(filterChanged(QString)),
             m_contactsListView, SLOT(setFilterString(QString)));
-
     connect(m_filterBar, SIGNAL(closeRequest()),
             m_filterBar, SLOT(hide()));
-
     connect(m_filterBar, SIGNAL(closeRequest()),
             m_searchContactAction, SLOT(trigger()));
 
-    connect(m_sortByPresenceAction, SIGNAL(activeChanged(bool)),
-            m_contactsListView, SLOT(toggleSortByPresence(bool)));
-
     connect(m_contactsListView, SIGNAL(genericOperationFinished(Tp::PendingOperation*)),
             this, SLOT(onGenericOperationFinished(Tp::PendingOperation*)));
 
@@ -325,8 +163,7 @@ void MainWidget::onAccountManagerReady(Tp::PendingOperation* op)
                            i18n("Something unexpected happened to the core part of your Instant Messaging system "
                            "and it couldn't be initialized. Try restarting the Contact List."),
                            i18n("IM system failed to initialize"));
-
-                           return;
+        return;
     }
 
     m_accountButtons->setAccountManager(m_accountManager);
@@ -354,7 +191,7 @@ void MainWidget::showMessageToUser(const QString& text, const MainWidget::System
 
 void MainWidget::onAddContactRequest()
 {
-    KTp::AddContactDialog *dialog = new KTp::AddContactDialog(m_contactsListView->accountsModel(), this);
+    KTp::AddContactDialog *dialog = new KTp::AddContactDialog(m_accountManager, this);
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->show();
 }
@@ -367,14 +204,13 @@ void MainWidget::onCustomContextMenuRequested(const QPoint &pos)
         return;
     }
 
-    Tp::ContactPtr contact;
-    QVariant item = index.data(AccountsModel::ItemRole);
+    KTp::RowType type = (KTp::RowType)index.data(KTp::RowTypeRole).toInt();
 
     KMenu *menu = 0;
 
-    if (item.userType() == qMetaTypeId<ContactModelItem*>()) {
+    if (type == KTp::ContactRowType) {
         menu = m_contextMenu->contactContextMenu(index);
-    } else if (item.userType() == qMetaTypeId<GroupsModelItem*>()) {
+    } else if (type == KTp::GroupRowType) {
         menu = m_contextMenu->groupContextMenu(index);
     }
 
@@ -394,7 +230,7 @@ void MainWidget::onGenericOperationFinished(Tp::PendingOperation* operation)
 
 void MainWidget::onJoinChatRoomRequested()
 {
-    QWeakPointer<JoinChatRoomDialog> dialog = new JoinChatRoomDialog(m_accountManager);
+    QWeakPointer<KTp::JoinChatRoomDialog> dialog = new KTp::JoinChatRoomDialog(m_accountManager);
 
     if (dialog.data()->exec() == QDialog::Accepted) {
         Tp::AccountPtr account = dialog.data()->selectedAccount();
@@ -473,9 +309,9 @@ void MainWidget::closeEvent(QCloseEvent* e)
 
 bool MainWidget::isPresencePlasmoidPresent() const
 {
-    QDBusInterface plasmoidOnDbus("org.kde.Telepathy.PresenceAppletActive", "/PresenceAppletActive");
+    QDBusReply<bool> serviceRegistered = QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.Telepathy.PresenceAppletActive");
 
-    if (plasmoidOnDbus.isValid()) {
+    if (serviceRegistered.isValid() && serviceRegistered.value()) {
         return true;
     } else {
         return false;
@@ -541,4 +377,203 @@ void MainWidget::toggleSearchWidget(bool show)
         }
 }
 
+void MainWidget::setupGlobalMenu()
+{
+    // Since our menu is hidden, its shortcuts do not work.
+    // Here's a workarond: we must assign global menu's unique
+    // items to main window. Since it's always active when an
+    // application is active, shortcuts now will work properly.
+
+    m_globalMenu = new KMenuBar(this);
+    m_globalMenu->setVisible(false);
+
+    KMenu *contacts = new KMenu(i18n("Contacts"), m_globalMenu);
+    contacts->addAction(m_addContactAction);
+    contacts->addAction(m_joinChatRoom);
+    if (!KStandardDirs::findExe("ktp-dialout-ui").isEmpty()) {
+        contacts->addAction(m_makeCall);
+    }
+    contacts->addAction(m_settingsDialog);
+    contacts->addSeparator();
+    contacts->addAction(m_quitAction);
+    this->addAction(m_quitAction); // Shortcuts workaround.
+    m_globalMenu->addMenu(contacts);
+
+    KMenu *view = new KMenu(i18n("View"), m_globalMenu);
+    view->addAction(m_groupContactsAction);
+    view->addAction(m_showOfflineAction);
+    view->addAction(m_sortByPresenceAction);
+    view->addSeparator();
+    KMenu *view_contactListTypeMenu = new KMenu(i18n("Contact List Type"), view);
+    view_contactListTypeMenu->addActions(m_contactListTypeGroup->actions());
+    view->addMenu(view_contactListTypeMenu);
+    KMenu *view_blockedFilterMenu = new KMenu(i18n("Shown Contacts"), view);
+    view_blockedFilterMenu->addActions(m_blockedFilterGroup->actions());
+    view->addMenu(view_blockedFilterMenu);
+    m_globalMenu->addMenu(view);
+
+    m_globalMenu->addMenu(helpMenu());
+}
+
+void MainWidget::setupToolBar()
+{
+    m_toolBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
+    m_toolBar->addAction(m_addContactAction);
+    m_toolBar->addAction(m_groupContactsAction);
+    m_toolBar->addAction(m_showOfflineAction);
+    m_toolBar->addAction(m_sortByPresenceAction);
+    m_toolBar->addAction(m_searchContactAction);
+
+    QWidget *toolBarSpacer = new QWidget(this);
+    toolBarSpacer->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
+    m_toolBar->addWidget(toolBarSpacer);
+
+    QToolButton *settingsButton = new QToolButton(this);
+    settingsButton->setIcon(KIcon("configure"));
+    settingsButton->setPopupMode(QToolButton::InstantPopup);
+
+    KMenu *settingsButtonMenu = new KMenu(settingsButton);
+    settingsButtonMenu->addAction(m_settingsDialog);
+
+    QActionGroup *delegateTypeGroup = new QActionGroup(this);
+    delegateTypeGroup->setExclusive(true);
+
+    KMenu *setDelegateTypeMenu = new KMenu(settingsButtonMenu);
+    setDelegateTypeMenu->setTitle(i18n("Contact List Type"));
+    setDelegateTypeMenu->addActions(m_contactListTypeGroup->actions());
+    settingsButtonMenu->addMenu(setDelegateTypeMenu);
+
+    KMenu *setBlockedFilterMenu = new KMenu(settingsButtonMenu);
+    setBlockedFilterMenu->setTitle(i18n("Shown Contacts"));
+    setBlockedFilterMenu->addActions(m_blockedFilterGroup->actions());
+    settingsButtonMenu->addMenu(setBlockedFilterMenu);
+
+    settingsButtonMenu->addAction(m_joinChatRoom);
+
+    if (!KStandardDirs::findExe("ktp-dialout-ui").isEmpty()) {
+        settingsButtonMenu->addAction(m_makeCall);
+    }
+
+    settingsButtonMenu->addSeparator();
+    settingsButtonMenu->addMenu(helpMenu());
+
+    settingsButton->setMenu(settingsButtonMenu);
+
+    m_toolBar->addWidget(settingsButton);
+}
+
+void MainWidget::setupTelepathy()
+{
+    Tp::registerTypes();
+    Tp::AccountFactoryPtr  accountFactory = Tp::AccountFactory::create(QDBusConnection::sessionBus(),
+                                                                       Tp::Features() << Tp::Account::FeatureCore
+                                                                       << Tp::Account::FeatureAvatar
+                                                                       << Tp::Account::FeatureCapabilities
+                                                                       << Tp::Account::FeatureProtocolInfo
+                                                                       << Tp::Account::FeatureProfile);
+
+    Tp::ConnectionFactoryPtr connectionFactory = Tp::ConnectionFactory::create(QDBusConnection::sessionBus(),
+                                                                               Tp::Features() << Tp::Connection::FeatureCore
+                                                                               << Tp::Connection::FeatureRosterGroups
+                                                                               << Tp::Connection::FeatureRoster
+                                                                               << Tp::Connection::FeatureSelfContact);
+
+    Tp::ContactFactoryPtr contactFactory = KTp::ContactFactory::create(Tp::Features()  << Tp::Contact::FeatureAlias
+                                                                      << Tp::Contact::FeatureAvatarToken
+                                                                      << Tp::Contact::FeatureAvatarData
+                                                                      << Tp::Contact::FeatureSimplePresence
+                                                                      << Tp::Contact::FeatureCapabilities
+                                                                      << Tp::Contact::FeatureClientTypes);
+
+    Tp::ChannelFactoryPtr channelFactory = Tp::ChannelFactory::create(QDBusConnection::sessionBus());
+    channelFactory->addFeaturesForTextChats(Tp::Features() << Tp::Channel::FeatureCore << Tp::TextChannel::FeatureMessageQueue);
+
+    m_accountManager = Tp::AccountManager::create(QDBusConnection::sessionBus(),
+                                                  accountFactory,
+                                                  connectionFactory,
+                                                  channelFactory,
+                                                  contactFactory);
+
+    connect(m_accountManager->becomeReady(), SIGNAL(finished(Tp::PendingOperation*)),
+            this, SLOT(onAccountManagerReady(Tp::PendingOperation*)));
+}
+
+KAction *MainWidget::createAction(const QString &text, QObject *signalReceiver, const char *slot, const KIcon &icon = KIcon())
+{
+    KAction *action = new KAction(icon, text, this);
+    connect(action, SIGNAL(triggered(bool)), signalReceiver, slot);
+    return action;
+}
+
+KAction *MainWidget::createAction(const QString& text, QObject *signalReceiver, const char* slot, bool isChecked, const KIcon& icon = KIcon())
+{
+    KAction *action = createAction(text, signalReceiver, slot, icon);
+    action->setCheckable(true);
+    action->setChecked(isChecked);
+    return action;
+}
+
+void MainWidget::setupActions(const KConfigGroup& guiConfigGroup)
+{
+    m_settingsDialog = KStandardAction::preferences(m_contactsListView, SLOT(showSettingsKCM()),this);
+    m_settingsDialog->setText(i18n("Settings")); // We set text manually since standard name is too long
+
+    m_quitAction = KStandardAction::quit(this, SLOT(close()), this);
+    m_quitAction->setMenuRole(QAction::QuitRole);
+
+    m_joinChatRoom = createAction(i18n("Join Chat Room..."), this, SLOT(onJoinChatRoomRequested()));
+    m_makeCall = createAction(i18n("Make a Call..."), this, SLOT(onMakeCallRequested()));
+    m_addContactAction = createAction(i18n("Add New Contacts..."), this, SLOT(onAddContactRequest()), KIcon("list-add-user"));
+    m_searchContactAction = createAction(i18n("Find Contact"), this, SLOT(toggleSearchWidget(bool)),
+                                         guiConfigGroup.readEntry("pin_filterbar", true), KIcon("edit-find-user"));
+    m_searchContactAction->setShortcut(KStandardShortcut::find());
+
+    // Dual actions
+    m_groupContactsAction = new KDualAction(i18n("Show contacts by groups"),
+                                            i18n("Show contacts by accounts"),
+                                            this);
+    m_groupContactsAction->setActiveIcon(KIcon("user-group-properties"));
+    m_groupContactsAction->setInactiveIcon(KIcon("user-group-properties"));
+    m_groupContactsAction->setCheckable(true);
+    m_groupContactsAction->setChecked(true);
+
+    m_showOfflineAction = new KDualAction(i18n("Show offline contacts"),
+                                          i18n("Show offline contacts"),
+                                          this);
+    m_showOfflineAction->setActiveIcon(KIcon("meeting-attending-tentative"));
+    m_showOfflineAction->setInactiveIcon(KIcon("meeting-attending-tentative"));
+    m_showOfflineAction->setCheckable(true);
+    m_showOfflineAction->setChecked(false);
+    m_showOfflineAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_S));
+
+    m_sortByPresenceAction = new KDualAction(i18n("Sorted by name"),
+                                             i18n("Sorted by presence"),
+                                             this);
+    m_sortByPresenceAction->setActiveIcon(KIcon("sort-presence"));
+    m_sortByPresenceAction->setInactiveIcon(KIcon("sort-name"));
+
+    // Setup contact list appearance
+    m_contactListTypeGroup = new QActionGroup(this);
+    m_contactListTypeGroup->setExclusive(true);
+    m_contactListTypeGroup->addAction(createAction(i18n("Use Full List"), m_contactsListView, SLOT(onSwitchToFullView()),
+                                                   guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("full")));
+    m_contactListTypeGroup->addAction(createAction(i18n("Use Normal List"), m_contactsListView, SLOT(onSwitchToCompactView()),
+                                                   guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("normal")
+                                                   || guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("compact"))); //needed for backwards compatibility
+
+    m_contactListTypeGroup->addAction(createAction(i18n("Use Minimalistic List"), m_contactsListView, SLOT(onSwitchToMiniView()),
+                                                   guiConfigGroup.readEntry("selected_delegate", "normal") == QLatin1String("mini")));
+
+    // Setup blocked contacts filtering
+    QString shownContacts = guiConfigGroup.readEntry("shown_contacts", "unblocked");
+    m_blockedFilterGroup = new QActionGroup(this);
+    m_blockedFilterGroup->setExclusive(true);
+    m_blockedFilterGroup->addAction(createAction(i18n("Show all contacts"), m_contactsListView, SLOT(onShowAllContacts()),
+                                                shownContacts == QLatin1String("all")));
+    m_blockedFilterGroup->addAction(createAction(i18n("Show unblocked contacts"), m_contactsListView, SLOT(onShowUnblockedContacts()),
+                                                shownContacts == QLatin1String("unblocked")));
+    m_blockedFilterGroup->addAction(createAction(i18n("Show blocked contacts"), m_contactsListView, SLOT(onShowBlockedContacts()),
+                                                shownContacts == QLatin1String("blocked")));
+}
+
 #include "main-widget.moc"
